#include <SoftwareSerial.h> //Δήλωση βιβλιοθήκης

const byte rxPin = 2; //Pin rx για επικοινωνία με HC-05
const byte txPin = 3; //Pin tx για επικοινωνία με HC-05
const byte ENAPin = 11; //Pin για Enable A H-Bridge PWM
const byte IN1Pin = 10; //Pin για IN1 H-Bridge
const byte IN2Pin = 9; //Pin για IN2 H-Bridge 
const byte LM35Pin = A0; //Pin αισθητήρα θερμοκρασίας

//Δημιουργία αντικειμένου για σειριακή επικοινωνία με HC-05
SoftwareSerial mySerial(rxPin, txPin); 
unsigned long lastReading = 0; //Χρόνος τελευταίας μέτρησης
boolean fw = true; //Μεταβλητή για τη φορά του μοτέρ

void setup()
{
  mySerial.begin(9600); //Αρχικοποίηση σειριακής επικοινωνίας
  analogReference(INTERNAL); //Τάση αναφοράς 1.1V για LM35
  pinMode(IN1Pin, OUTPUT); //Τα pin για τη φορά του μοτέρ
  pinMode(IN2Pin, OUTPUT); //ορίζονται ως έξοδοι
  digitalWrite(IN1Pin, HIGH); //Ορισμός αρχικής φοράς μοτέρ 
  digitalWrite(IN2Pin, LOW);
  analogWrite(ENAPin, 0); //Ταχύτητα μοτέρ 0
}

void loop()
{
  byte motorSpeed = 0; //Μεταβλητή για ταχύτητα μοτέρ
  float temperature; //Μεταβλητή για υπολογισμό θερμοκρασίας
  byte command; //Τύπος εισερχόμενης εντολής από το BT

  //Αν έχει περάσει 1 sec από την προηγούμενη μέτρηση
  if (millis() - lastReading >= 1000) 
  {
    //Ενημέρωση χρόνου τελευταίας μέτρησης
    lastReading = millis(); 
    //Υπολογισμός θερμοκρασίας
    temperature = (analogRead(LM35Pin) * 1.1 * 100 / 1024);
    mySerial.println(temperature); //Αποστολή θερμοκρασίας
  }
  if (mySerial.available() > 0) 
  { //Αν υπάρχουν εισερχόμενα δεδομένα
    command = mySerial.read(); //Διάβασε κωδικό εντολής
    if (command == 1) //Εντολή 1: ορισμός νέας ταχύτητας
    {
      //Αναμονή μέχρι να ληφθεί 2ο byte
      while(mySerial.available()==0); 
      motorSpeed = mySerial.read(); //Διάβασε την ταχύτητα
      //Γράψε την ταχύτητα στο μοτέρ
      analogWrite(ENAPin, motorSpeed); 
    }
    else if (command == 2) //Εντολή 2: αλλαγή φοράς
    {
      reverse(); //Κλήση διαδικασίας για αλλαγή φοράς
    }
  }
}

void reverse() //Διαδικασία αλλαγής φοράς
{
  if (fw) //Αν η φορά είναι προς τα εμπρός
  {
    digitalWrite(IN1Pin, LOW); //Αντίστροφη φορά
    digitalWrite(IN2Pin, HIGH);
  }
  else //Αλλιώς
  {
    digitalWrite(IN2Pin, LOW); //Φορά προς τα εμπρός
    digitalWrite(IN1Pin, HIGH);
  }
  fw = !fw; //Ενημέρωση τιμής της fw
}

